/*
 * DHT11 温湿度センダー
 * 
 */

#include "DHT11.h"

#define DATAPIN 25 // コレは何番ピンでも良い

/**
 * DHT11初期化
 */
void DHT11::init( ) {
  pinMode(DATAPIN, INPUT_PULLUP);

  dht11.humidityN = 8 ;
  dht11.humidityD = 0 ;
  dht11.temperatureN = 8 ;
  dht11.temperatureD = 8 ;
}

/*
 * 温湿度読み取り
 * 
 * @return true - 読み取り成功 / false - 読み取り失敗
 */
bool DHT11::readDTH11( ) {
  uint8_t readData[5] ;
  uint8_t checkSum = 0 ;
  int setPos = 0 ;
  int bitLen = 0 ;
  int stat = HIGH ;
  uint8_t bitData ;
  unsigned long beginTime ;
  unsigned long passTime ;
  unsigned long zeroTime = 35 ; // 35μ秒以下なら０ 以上なら１と判定 ※仕様では0は26～28μ秒
  bool flagStart = false;

  for (int i=0;i<5;i++) {
    readData[i] = 0 ;
  }

  // ----- データ送信要求 -----
  pinMode(DATAPIN, OUTPUT); //開始信号
  delay(20);
  pinMode(DATAPIN, INPUT);
  while(digitalRead(DATAPIN) == LOW) {
    delayMicroseconds(5) ;
  }

  // ----- ACSEPT -----
  flagStart = false;
  passTime = 0 ;
  beginTime = micros();
  while (passTime < 1000) {
    passTime = micros() - beginTime ;
    if (digitalRead(DATAPIN) !=  stat) {
      if (stat == HIGH) {
        stat = LOW ;
        if (flagStart) {
           break ; 
        }
      } else {
        beginTime = micros();
        flagStart = true ;
        stat = HIGH ;
      }
    }
  }
  
  // ----- データ受信 -----
  bitData = 0x80 ;
  flagStart = false;
  passTime = 0 ;
  beginTime = micros();
  while (bitLen < 40 && passTime < 1000) {
    passTime = micros() - beginTime ;
    if (digitalRead(DATAPIN) !=  stat) {
      if (stat == HIGH) {
        stat = LOW ;
        if (flagStart) {
          // ----- 1Bit 受信 -----
          if (passTime > zeroTime) {
            readData[setPos] |= bitData ;
          }
          if (bitData != 0x01) {
            bitData >>= 1 ;
          } else {
            bitData = 0x80 ;
            setPos ++ ;
          }

          // ----- ステート変数リセット -----
          beginTime = micros();
          flagStart = false;
          bitLen ++ ;
        }
      } else {
        beginTime = micros();
        flagStart = true ;
        stat = HIGH ;
      }
    }
  }

  // ----- 取得データ格納 -----
  dht11.humidityN = readData[0] ;   // 湿度の整数部 Range:5~95
  dht11.humidityD = readData[1] ;   // 湿度の小数部 ※は常に0
  dht11.temperatureN = readData[2]; // 温度の整数部 -20 ～ 
  dht11.temperatureD = readData[3]; // 温度の小数部
  checkSum = readData[0] + readData[1] + readData[2] + readData[3] ;

  return (bitLen == 40 && checkSum == readData[4]) ;
}
