/*
 * LPS25HB 気圧センサー
 * 
 */

#include <Wire.h>
#include <arduino.h>
#include "LPS25HB.h"
#define LPSADDR 0x5C

/**
 * DHT11初期化
 */
bool LPS25HB::init(){
  Wire.begin();
  lps25hb.pressure = 0 ;
  lps25hb.temperature = 0.0 ;

  // -- Read WHO AM I
  uint8_t deviceID = readLPS(0x0F) ;
  if (deviceID != 0xBD) {
    return false ;
  }

  // --- Reset LPS25HB
  sendLPS(0x21,0x04);   // ソフトウェアリセット
  while(readLPS(0x21)); // リセット確認
  delay(100);
  sendLPS(0x21,0x80);   // リブートメモリ
  while(readLPS(0x21)); // リブート確認
  delay(100);
  sendLPS(0x20,0xc0);   // アクティブモード、アウトプット25Hz
                        // ※VCCに5V入れた場合、1Hzだと何故か誤差が大きい
                        // ※この設定だと+2hPa +1℃位のズレで収まる
  delay(100);
  sendLPS(0x21,0x40);   // FiFO有効
  sendLPS(0x2E,0xC7);   // FIFO平均モード 8サンプル
                        // ※サンプル数が大きすぎるとFIFOがFULLになるまでに時間がかかる
  delay(1300);

  return true ;
}

// Register 1Byte 読み取り
uint8_t LPS25HB::readLPS(uint8_t addr) {
  uint8_t readData ;
  // Read Address Set
  Wire.beginTransmission(LPSADDR);
  Wire.write(addr);
  Wire.endTransmission();
  
  // 1Byte Read
  Wire.requestFrom(LPSADDR, 1);
  while(Wire.available()<1);
  readData = Wire.read();

  return readData ;
}

// Register n Byte 読み取り
uint8_t LPS25HB::readLPSBuff(uint8_t addr,int readLen, uint8_t *i2cBuff) {
  // Read Address Set
  Wire.beginTransmission(LPSADDR);
  Wire.write(addr|0x80);
  Wire.endTransmission();
  
  // 1Byte Read
  Wire.requestFrom(LPSADDR , readLen);
  while(Wire.available() < readLen);
  for (int i=0;i<readLen;i++) {
    i2cBuff[i] = Wire.read();
  }
}

// 1バイトコマンドを送信
void LPS25HB::sendLPS(uint8_t addr , uint8_t command) {
  Wire.beginTransmission(LPSADDR);
  Wire.write(addr);
  Wire.write(command);
  Wire.endTransmission();
}

// 気圧センサー読み取り
bool LPS25HB::readLPS25HB( ) {
  uint8_t stat = readLPS(0x27);
  uint8_t pxl = readLPS(0x28) ;
  uint8_t pl = readLPS(0x29) ;
  uint8_t ph = readLPS(0x2A) ;
  uint8_t tl = readLPS(0x2B) ;
  uint8_t th = readLPS(0x2C) ;
  uint8_t fifoStat = readLPS(0x2F);

  // combine bytes
  lps25hb.pressure = (int32_t)(int8_t)ph << 16 | (uint16_t)pl << 8 | pxl;
  lps25hb.pressure /= 4096 ;
  int16_t temp = (int16_t)th << 8 | tl ;
  lps25hb.temperature = 42.5 + (float)temp / 480;

  return ((stat & 0x03) == 0x03) && ((fifoStat & 0xC0) == 0xC0)  ;
}
