/*
 * M5Stamp PICO IoT実験
 * 温湿度をサーバに登録するテスト
 */
#include <WiFi.h>
#include <Wire.h>
#include "Ambient.h"
#include "DISP7SEG.h"
#include "DHT11.h"
#include "LPS25HB.h"

#define DEBUG true
#define BAUDRATE 9600
#define LED_PIN 26
#define OLED_SWITCH_PIN 19
char str[120] ;
bool useOled = true ;
bool useLPS25HB = true ;

// --- 
DISP7SEG disp ;   // OLED
DHT11 dht11 ;     // DHT11
LPS25HB lps25hb ; // LPS25HB

// ----- NetWork Data ---
const char* ssid     = "****";
const char* password = "********";

// --- 時刻調整用
unsigned long passTime;
int sendTime = 0 ;

// --- Ambient接続用
#define CH_NO 0 // 送信間隔を開けないとAmbieent側でリクエストを受け付けない…気がするのでタイミングをずらしてみる
#define CH_TMP 1
#define CH_HUM 2
#define CH_PHA 3
WiFiClient client;
Ambient ambient;
unsigned int channelId =  12345 ; // AmbientのチャネルID
const char* writeKey = "*******"; // ライトキー

// -------------------------------
// ----- データ送信タイミング取得 ----
// --------------------------------
void connectWiFi( ) {
  if (WiFi.status() != WL_CONNECTED) {
    WiFi.reconnect( ) ;
    int i = 0 ;
    while (WiFi.status() != WL_CONNECTED) {
      delay(500);
#if DEBUG
      Serial.print(".");
#endif
      i++ ;
      if (i > 20) {
        delay(30000);
        return  ;
      }
    }
#if DEBUG      
    Serial.println("");
    Serial.println("WiFi connected");
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
#endif
  }
}

int getSpan( ) {
  connectWiFi() ;
  configTime(9 * 3600L, 0, "ntp.nict.jp", "time.google.com", "ntp.jst.mfeed.ad.jp");
  struct tm timeInfo;
  getLocalTime(&timeInfo) ;
  sendTime = timeInfo.tm_min / 5 ;
  passTime = millis();
#if DEBUG      
  Serial.print(timeInfo.tm_hour) ;
  Serial.print(":") ;
  Serial.print(timeInfo.tm_min) ;
  Serial.print(":") ;
  Serial.println(timeInfo.tm_sec) ;
  Serial.println("Timing Reset - 3") ;
#endif
  return (5 - (timeInfo.tm_min % 5)) * 60 -  timeInfo.tm_sec + CH_NO * 30 ;
}

int getWaitTime() {
  int waitTime = 5 * 60 ;
  unsigned long nowTime = millis();
  if (nowTime < passTime) {
    // ----- リセット掛かってる
    waitTime = getSpan( ) ;
  } else {
    waitTime = (5 * 60) - ( ((nowTime - passTime) / 1000) % (5 * 60)) +  + CH_NO * 30 ;
#if DEBUG
    Serial.print("waitTime = ") ;
    Serial.println(waitTime) ;
    Serial.print("nowTime = ") ;
    Serial.println(nowTime) ;
    Serial.print("passTime = ") ;
    Serial.println(passTime) ;
#endif
  }
  return waitTime ;
}

// ----------------------
// ----- セットアップ -----
// ----------------------
void setup() {
  pinMode(LED_PIN,OUTPUT) ;
  digitalWrite(LED_PIN,HIGH) ;
  pinMode(OLED_SWITCH_PIN,INPUT) ;
  if (digitalRead(OLED_SWITCH_PIN) == HIGH) {
    useOled = true ;
  }

#if DEBUG
  Serial.begin(BAUDRATE);
  Serial.println("\nWakeUp") ;
  Serial.print("useOled=") ;
  Serial.println(useOled) ;
  Serial.print("useLPS25HB=") ;
  Serial.println(useLPS25HB) ;
#endif
  if (useOled) {
    disp.init() ;
  }

  if (useLPS25HB) {
    Serial.print("lps25hb.init( )") ;
    useLPS25HB = lps25hb.init( );
    Serial.println(" - Ready") ;
  }

  // --- Netwaorkに接続
  Serial.println("Netwaorkに接続") ;
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
#if DEBUG
    Serial.print(".");
#endif
  }
#if DEBUG
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
#endif
  ambient.begin(channelId, writeKey, &client); // チャネルIDとライトキーを指定してAmbientの初期化

  // --- 起動表示---
  if (useOled) {
    disp.disp7segLEDP( 0 , 0 , 0 ) ;
  }

  // --- 5分刻み毎の動作になるようタイミング調整のスリープ
  int s = getSpan();
#if DEBUG
  Serial.print("Wait ");
  Serial.print(s);
  Serial.println(" Sec");
#endif
  for (int i=0;i<s;i++) {
    digitalWrite(LED_PIN,(((i%2)==0)?LOW:HIGH)) ;
    delay(1000) ;
  }
  digitalWrite(LED_PIN,LOW) ;
}

// ----------------------
// ----- ループ-----
// ----------------------
void loop() {
  // ----- Ambient登録用
  float temp, humid ;
  int pressure ;

  // ----- 温湿度読み取り -----
  bool ret = dht11.readDTH11( );
#if DEBUG
  Serial.print("dht11.readDTH11( ) = ") ; Serial.println((ret?"TRUE":"FALSE")) ;
#endif
  while(!ret) {
    // 読み取り失敗：3秒 Wait
    delay(3000) ;
    ret = dht11.readDTH11( );
#if DEBUG
    Serial.print("dht11.readDTH11( ) = ") ; Serial.println((ret?"TRUE":"FALSE")) ;
#endif
  }

  if (useLPS25HB) {
    ret = lps25hb.readLPS25HB( );
#if DEBUG
  Serial.print("lps256hb.readLPS25HB( ) = ") ; Serial.println((ret?"TRUE":"FALSE")) ;
#endif
    while(!ret) {
      // 読み取り失敗：3秒 Wait
      delay(3000) ;
      ret = lps25hb.readLPS25HB( );
#if DEBUG
      Serial.print("dht11.readDTH11( ) = ") ; Serial.println((ret?"TRUE":"FALSE")) ;
#endif
    }
  }

  // --- 7セグメントLEDっぽい表示 ---
  if (useOled) {
//    disp.disp7segLED( dht11.getHumidityN() ,dht11.getTemperatureN() ,dht11.getTemperatureD() ) ;
    disp.disp7segLEDP( lps25hb.getPressure() ,lps25hb.getTemperatureN() ,lps25hb.getTemperatureD() ) ;
  }

  // ----- Ambientに登録 -----
  humid = (float)dht11.getHumidityN();
  temp  = dht11.getTemperatureN() + dht11.getTemperatureD() / 10. ; 
#if DEBUG
  Serial.print("temp: "); Serial.println(String(temp).c_str());
  Serial.print("humid:") ; Serial.println(String(humid).c_str());
#endif

  // 温度、湿度の値をAmbientに送信する
  connectWiFi( ) ;
  ambient.set(CH_TMP, String(temp).c_str());
  ambient.set(CH_HUM, String(humid).c_str());
  if (useLPS25HB) {
    pressure = (int)lps25hb.getPressure() ;
#if DEBUG
  Serial.print("pressure:") ; Serial.println(String(pressure).c_str());
#endif
    ambient.set(CH_PHA, String(pressure).c_str());
  }
  ambient.send();

  // ----- 5分間隔になるように時間調整
  int s = 5 * 60 ;
  sendTime = (sendTime + 1) % 12 ;  
  if (sendTime == 0) {
    // --- 1時間に1回　NTPから現在時刻を取得してタイマーを再設定
#if DEBUG
    Serial.println("Timing Reset - 1");
#endif
    s = getSpan();
  } else {
#if DEBUG
    Serial.println("Timing Reset - 2");
#endif
    s = getWaitTime();
  }
#if DEBUG
  Serial.print("sendTime ");
  Serial.println(sendTime);
  Serial.print("Wait ");
  Serial.print(s);
  Serial.println(" Sec");
#endif
  delay(s*1000) ; // 5分間隔Wait
}
